/*
 * Interface program to the HP PhotoSmart camera -- M. Demleitner 1998,99,2000
 * This program grew out of the program I tried my hypotheses on.  That's one 
 * reason why it's such a mess.  Hope it works for you.  Read README and drop 
 * me a note at msdemlei@tucana.harvard.edu 
 */

/*
 * Changelog:
 *  See changes.txt
 * 
 * 0.93 (unreleased):
 * . split source into multiple files: common source and user apps (JS)
 * . added thumbnail support (JS)
 * . wishlist: when we download a jpg (not a thumbnail), output a
 *   text file (<filename>.txt) that lists when the picture was taken,
 *   along with any other info we wanna put there. I'd like it. Otherwise,
 *   the timestamp on the file is the only way to tell (and that's when it
 *   was downloaded, not taken). (JS)
 * 
 * 
 * 0.92: hacked mercilessly by Jeff to get it to work with his C30
 * . added a bunch of prettying-up PROTOSNOOPs
 *   -- these could maybe go away after a bit?
 * . added function prototypes removed a few unused functions (keep around 
 *   somewhere?)
 *   -- delay(), hexput(), camflush(), camputs()
 * . rewrote send_escaped()
 * . send_command : get_datablock length is sometimes 10 for me
 *   -- this fixes a bunch of stuff for my C30
 * . changed the wording of a few * "picno" "imgid" to maybe? clear up
 *   some confusion
 * . hacked the dirall() formatting
 *   -- added date/time
 * . changed for() loop start from 0 to 1 for dirall() and getall()
 * . added a utility function: decode_str(); useful when printing ascii
 *   data out of a data block
 * . added some error checking in main(), when parsing image numbers
 * . changed Makefile to have 'uninstall' undo what 'install' does
 * . wishlist -- thumbnails?
 *
 * 0.91: (MD)
 * . dtr_raise/dtr_drop should now work without taking recourse to
 *   voodoo (set functions take a pointer)
 * . took out the fancy poll mechanism to eliminate header problem
 *   with glibc 2.1 
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>
#include <sys/poll.h>
#include <sys/ioctl.h>
#include <signal.h>
#include "psm.h"

#ifdef PSM_GLIBC
#include <errno.h>
#include <termios.h>
#endif

int debug = (MASTERDEBUG || 0);
int protosnoop = (MASTERDEBUG || 0);
int sendsnoop = (MASTERDEBUG || 0);

int _camgetc_cache = -1;
int byte_count = 0;
char *versionstring = "psmget version " PSM_VERSION;

/*
 * Need this in the signal handler 
 */
static int global_cam = -1;

void commchar_log(FILE * file, char dir, uchar c)
{
   switch (c) {
   case STX:
      fprintf(file, " %cSTX", dir);
      break;
   case ETX:
      fprintf(file, " %cETX", dir);
      break;
   case EOT:
      fprintf(file, " %cEOT", dir);
      break;
   case ENQ:
      fprintf(file, " %cENQ", dir);
      break;
   case ACK:
      fprintf(file, " %cACK", dir);
      break;
   case NACK:
      fprintf(file, " %cNACK", dir);
      break;
   case ETB:
      fprintf(file, " %cETB", dir);
      break;
   case '\n':
      fprintf(file, "\n");
      break;
   default:
      fprintf(file, " %c%03x", dir, c);
   }
}

int parse_numbers(char *incoming, int max, char *array)
{
   int num, start, stop, count;
   char *c;

   /* poor man's strtok */
   c = strchr(incoming, ',');
   if (NULL != c) {
      *c = '\0';
      ++c;
      return parse_numbers(incoming, max, array) +
	  parse_numbers(c, max, array);
   } else {
      if (2 <= sscanf(incoming, "%d-%d%n", &start, &stop, &count)) {
	 if (stop < start) {
	    fprintf(stderr,
		    "Error: end of range is before "
		    "beginning of range: %s\n", incoming);
	    return 1;
	 }
	 if (count != strlen(incoming)) {
	    fprintf(stderr, "Cruft at end of range string.\n");
	    return 1;
	 }
	 if (stop < 0 || start < 0) {
	    fprintf(stderr, "We don't like negative numbers.\n");
	    return 1;
	 }
	 if (stop > max) {
	    fprintf(stderr,
		    "Top range is too big; truncating at %d\n", max);
	    stop = max;
	 }
	 for (num = start; num <= stop; num++) {
	    array[num] = 1;
	 }

      } else {
	 if (1 <= sscanf(incoming, "%d%n", &num, &count)) {
	    if (count != strlen(incoming)) {
	       fprintf(stderr, "Cruft at end of number.\n");
	       return 1;
	    }
	    if (num < 0) {
	       fprintf(stderr, "We don't like negative numbers.\n");
	       return 1;
	    }
	    if (num > max) {
	       fprintf(stderr, "%d is too big; truncating at %d\n",
		       num, max);
	       num = max;
	    }
	    array[num] = 1;
	 } else {
	    fprintf(stderr, "Couldn't match %s\n", incoming);
	 }
      }
   }

   return 0;
}

void camungetc(uchar c)
{
   _camgetc_cache = c;
}

/*
 * get a character from the camera and wait timeout ms.  If timeout occurs,
 * return -1, otherwise the char 
 */
int camgetc(int cam, int timeout)
{
   uchar ch = 0xfc;
   int t = 0;

   if (_camgetc_cache != -1) {
      ch = _camgetc_cache;
      _camgetc_cache = -1;
      return ch;
   }
   while (1 != read(cam, &ch, 1) && t <= timeout) {
      usleep(SLEEPLEN * 1000U);
      t += SLEEPLEN;
   }
   if (t > timeout)
      return -1;
   if (ch == 0x1b) {
      t = 0;
      while (1 != read(cam, &ch, 1) && t <= timeout) {
	 usleep(SLEEPLEN * 1000U);
	 t += SLEEPLEN;
      }
      if (t > timeout)
	 return -1;
      ch = ~ch;
      PROTOSNOOP(stderr, '<', ch);
   } else {
      PROTOSNOOP(stderr, '<', ch);
   }
   if (++byte_count % 15 == 0) {
      PROTOSNOOP(stderr, '\n', '\n');
   }
   return ch;
}

int camputc(int cam, uchar ch)
{
   int wstat;
   int numtry = 10;

   while ((1 != (wstat = write(cam, &ch, 1))) && numtry--)
      usleep(SLEEPLEN);
   if (!numtry)
      fprintf(stderr,
	      "Write failure on serial port.  Expect the worst.\n");
   SENDSNOOP(stderr, '>', ch);
   return 0;
}

/*
 * Get a data block from the camera, return NULL on timeout, (void*)(-1) on
 * protocol error (this might not be a good idea, but should work on all
 * machines linux is likely to work on), otherwise a pointer to a freshly
 * allocated block of memory with the data in it.
 */
uchar *get_datablock(int cam, int *len, int *etbetx, int retry,
		     int init_timeout)
{
   int a, b, c;
   uchar *buf, *bp;
   int i;
   uchar checksum = 0;

   DEBUG(stderr, "Entering get_datablock\n");
   if (init_timeout)
      c = camgetc(cam, init_timeout);
   else
      c = camgetc(cam, TIMEOUT_EOT);
   PROTOSNOOP(stderr, '\n', '\n');
   if (c != STX) {
      if (c == -1)
	 if (retry < 3)
	    return get_datablock(cam, len, etbetx, retry + 1, 0);
	 else
	    return (void *) (-1);
      else if (c != ENQ)
	 return (void *) (-1);
      camputc(cam, ACK);
      PROTOSNOOP(stderr, '\n', '\n');
      c = camgetc(cam, TIMEOUT_WAITDATA);
      PROTOSNOOP(stderr, '\n', '\n');
      if ((c != STX) && (c != -1))
	 return (void *) (-1);
   }
   if (c != -1) {
      a = camgetc(cam, TIMEOUT_WAITDATA);
      PROTOSNOOP(stderr, '\n', '\n');
      if (a != -1)
	 b = camgetc(cam, TIMEOUT_WAITDATA);
      PROTOSNOOP(stderr, '\n', '\n');
      if (c == -1 || a == -1 || b == -1) {
	 if (retry < 3) {
	    fprintf(stderr, "T");
	    camputc(cam, NACK);
	    PROTOSNOOP(stderr, '\n', '\n');
	    return get_datablock(cam, len, etbetx, retry + 1, 0);
	 } else
	    return NULL;
      }
   }
   checksum = (unsigned char) (a + b);
   i = *len = a + b * 0x100;
   DEBUG(stderr, "\nGetting data block of length %d\n", i);
   bp = buf = malloc(*len);
   while (i--) {
      c = camgetc(cam, TIMEOUT_WAITDATA);
      if (c == -1) {
	 if (retry < 3) {
	    fprintf(stderr, "T");
	    camputc(cam, NACK);
	    PROTOSNOOP(stderr, '\n', '\n');
	    return get_datablock(cam, len, etbetx, retry + 1, 0);
	 } else
	    return NULL;
      }
      checksum += c;
      *bp++ = c;
   }
   PROTOSNOOP(stderr, '\n', '\n');

   a = camgetc(cam, TIMEOUT_WAITDATA);
   if (a != -1) {
      b = camgetc(cam, TIMEOUT_WAITDATA);
      checksum += a;
   }
   PROTOSNOOP(stderr, '\n', '\n');
   if (b != checksum)
      fprintf(stderr, "C");
   if (a == -1 || b == -1 || b != checksum) {
      DEBUG(stderr,
	    "Error while reading checksum or checksum error (%x).\n",
	    checksum);
      if (retry < 3) {
	 camputc(cam, NACK);
	 PROTOSNOOP(stderr, '\n', '\n');
	 return get_datablock(cam, len, etbetx, retry + 1, 0);
      } else
	 return NULL;
   }
   camputc(cam, ACK);
   PROTOSNOOP(stderr, '\n', '\n');
   c = camgetc(cam, TIMEOUT_EOT);
   PROTOSNOOP(stderr, '\n', '\n');
   if (c == -1) {
      b = camgetc(cam, TIMEOUT_EOT);
      PROTOSNOOP(stderr, '\n', '\n');
      if (b == -1)
	 return (void *) (-1);
      else if (b == STX) {
	 camputc(cam, NACK);
	 if (retry < 3)
	    return get_datablock(cam, len, etbetx, retry + 1, 0);
	 else
	    return (void *) (-1);
      } else {
	 camungetc(b);
	 if (retry < 3)
	    return get_datablock(cam, len, etbetx, retry + 1, 0);
	 else
	    return (void *) (-1);
      }
   }
   *etbetx = a;
   return buf;
}

int camreaddata(int cam, FILE * ausf)
{
   int len, etbetx;
   uchar *buf;

   DEBUG(stderr, "Entering camreaddata\n");
   while (1) {
      buf = get_datablock(cam, &len, &etbetx, 0, 0);
      if (buf == NULL) {
	 fprintf(stderr, "Irreparable Timeout.\n");
	 return -1;
      }
      if (buf == (void *) (-1)) {
	 fprintf(stderr, "Protocol Error.\n");
	 return -1;
      }
      fprintf(stderr, "#");
      DEBUG(stderr, "Read %d bytes\n", len);
      fwrite(buf, len, 1, ausf);
      free(buf);
      if (etbetx == ETX)
	 return 0;
   }
   return -1;
}

int send_escaped(int cam, uchar c)
{
   switch (c) {
   case 0x02:
   case 0x11:
   case 0x13:
   case 0x1b:
   case 0x1d:
      camputc(cam, 0x1b);
      camputc(cam, ~c);
      break;
   default:
      camputc(cam, c);
      break;
   }
   return 0;
}

int camsenddata(int cam, uchar * data, int len)
{
   int c;
   uchar checksum = 0;
   int retries = 0;

   camputc(cam, ENQ);
   while (retries++ < 3) {
      c = camgetc(cam, TIMEOUT_WAITACK);
      PROTOSNOOP(stderr, '\n', '\n');
      if (c == ACK)
	 break;
      camputc(cam, ENQ);
   }
   if (retries == 4)
      return -1;
   retries = 0;
   camputc(cam, STX);
   camputc(cam, (uchar) (len & 0xff));
   camputc(cam, (uchar) ((len >> 8) & 0xff));
   checksum += (uchar) (len & 0xff) + (uchar) ((len >> 8) & 0xff);
   while (len-- > 0) {
      checksum += *data;
      send_escaped(cam, *data++);
   }
   camputc(cam, ETX);
   checksum += ETX;
   send_escaped(cam, checksum);
   c = camgetc(cam, TIMEOUT_ENDDATA);
   PROTOSNOOP(stderr, '\n', '\n');
   if (c == -1)
      return -1;
   if (c == NACK)
      return camsenddata(cam, data, len);
   if (c != ACK)
      return -1;
   camputc(cam, EOT);
   return 0;
}

/*
 * Test if camera is there 
 */
int ping(int cam, int retries)
{
   int c;

   retries++;
   while (--retries) {
      camputc(cam, ENQ);
      c = camgetc(cam, TIMEOUT_WAITACK);
      PROTOSNOOP(stderr, '\n', '\n');
      if (c == ACK)
	 break;
   }
   if (!retries) {
      DEBUG(stderr, "\nPing failed\n");
      return -1;
   }
   camputc(cam, EOT);
   DEBUG(stderr, "\nPing successful\n");
   return 0;
}

int raise_dtr(int cam)
{
   int stat;

   ioctl(cam, TIOCMGET, &stat);
   stat |= TIOCM_DTR;
   if (ioctl(cam, TIOCMSET, &stat) < 0) {
      fprintf(stderr, "Error raising DTR.  Errno %d\n", errno);
      exit(-1);
   }
   ioctl(cam, TIOCMGET, &stat);
   return 0;
}

int drop_dtr(int cam)
{
   int stat;

   ioctl(cam, TIOCMGET, &stat);
   stat &= ~TIOCM_DTR;
   ioctl(cam, TIOCMSET, &stat);
   ioctl(cam, TIOCMGET, &stat);
   return 0;
}

/*
 * report a camera error 
 */
void cam_err(int errcode)
{
   char *errstr;

   switch (errcode) {
   case 0x0101:
      errstr = "Focusing Malfunction";
      break;
   case 0x0102:
      errstr = "Iris Malfunction";
      break;
   case 0x0201:
      errstr = "Strobe charging problem";
      break;
   case 0x0203:
      errstr = "EEPROM checksum error";
      break;
   case 0x0205:
      errstr = "Timing signal error (DSP)";
      break;
   case 0x0206:
      errstr = "Timing signal error (JPEG)";
      break;
   case 0x0301:
      errstr = "Miniature card not present";
      break;
   case 0x0311:
      errstr = "Miniature card not supported";
      break;
   case 0x0321:
      errstr = "Miniature card removed during access";
      break;
   case 0x0340:
      errstr = "Miniature card cannot be read";
      break;
   case 0x0341:
      errstr = "Miniature card cannot be written";
      break;
   case 0x0381:
      errstr = "Miniature card is write protected";
      break;
   case 0x0382:
      errstr = "Miniature card is full";
      break;
   case 0x0401:
      errstr = "Light condition too dark";
      break;
   case 0x0402:
      errstr = "AF cannot be performed";
      break;
   case 0x0501:
      errstr = "System error";
      break;
   case 0x0800:
      errstr = "Illegal parameter";
      break;
   case 0x0801:
      errstr = "Cannot cancel";
      break;
   case 0x0C01:
      errstr = "Unsupported command";
      break;
   case 0x0C02:
      errstr = "Other command executing";
      break;
   case 0x0C03:
      errstr = "Command order error";
      break;
   case 0x0FFF:
      errstr = "Unknown error";
      break;
   default:
      fprintf(stderr, "%x ", errcode);
      errstr = "Camera on fire -- please report";
      break;
   }
   fprintf(stderr, "%s\n", errstr);
}

int cam_open(char *devnam, int bps)
/*
 * bps: one of 19200 38400 57000 115200 
 */
{
   int ser;
   struct termios termc;
   uchar setspeed[] =
       { 0x80, 0x90, 0x00, 0x00, 0x40, 0x00, 0x01, 0x00 };
   int dum, dumint;
   int flags;
   int bitrates[] = {
      9600, B9600,
      19200, B19200,
      38400, B38400,
      57600, B57600,
      115200, B115200,
      0
   };
   int *cbitr = bitrates;
   uchar *resblock;
   int retries = 0;
   int res;

   ser = open(devnam, O_RDWR | O_NOCTTY);
   if (ser < 0)
      return 0;
   flags = fcntl(ser, F_GETFL);
   flags |= O_NONBLOCK;
   fcntl(ser, F_SETFL, flags);
   tcgetattr(ser, &termc);
   cfmakeraw(&termc);
   termc.c_iflag |= IGNBRK;
   termc.c_iflag |= IXON;
   termc.c_iflag |= IXOFF;
   termc.c_lflag &= !ECHO;
   cfsetospeed(&termc, cbitr[1]);
   cfsetispeed(&termc, cbitr[1]);
   tcsetattr(ser, TCSANOW, &termc);
   raise_dtr(ser);
   do {
      DEBUG(stderr, "Trying to connect at %d\n", *cbitr);
      if (ping(ser, 1)) {
	 cbitr += 2;
	 if (*cbitr == 0)
	    cbitr = bitrates;
	 cfsetospeed(&termc, cbitr[1]);
	 cfsetispeed(&termc, cbitr[1]);
	 res = tcsetattr(ser, TCSANOW, &termc);
	 if (res < 0)
	    fprintf(stderr,
		    "Could not set serial speed.  Please report\n");
      } else
	 break;
      retries++;
   }
   while (retries < 10);
   if (retries == 10) {
      drop_dtr(ser);
      return -1;
   }
   fprintf(stderr, "Found camera at %d bps\n", *cbitr);
   if (bps == 0 || bps == *cbitr) {
      global_cam = ser;
      return ser;
   }

   fprintf(stderr, "Trying to set camera to %d bps...\n", bps);
   switch (bps) {
   case 19200:
      break;
   case 38400:
      setspeed[4] = 0x80;
      break;
   case 57600:
      setspeed[4] = 0;
      setspeed[5] = 1;
      break;
   case 115200:
      setspeed[4] = 0;
      setspeed[5] = 2;
      break;
   default:
      return ser;
   }
   camsenddata(ser, setspeed, 8);
   resblock = get_datablock(ser, &dum, &dumint, 0, 0);
   if (resblock && resblock != (void *) (-1))
      free(resblock);
   else
      return -1;
   switch (bps) {
   case 19200:
      cfsetospeed(&termc, B19200);
      cfsetispeed(&termc, B19200);
      break;
   case 38400:
      cfsetospeed(&termc, B38400);
      cfsetispeed(&termc, B38400);
      break;
   case 57600:
      cfsetospeed(&termc, B57600);
      cfsetispeed(&termc, B57600);
      break;
   case 115200:
      cfsetospeed(&termc, B115200);
      cfsetispeed(&termc, B115200);
      break;
   }
   tcsetattr(ser, TCSADRAIN | TCSAFLUSH, &termc);
   close(ser);
   return cam_open(devnam, bps);
}

int cam_close(int cam)
{
   DEBUG(stderr, "Closing connection to camera\n");
   drop_dtr(cam);
   close(cam);
   global_cam = -1;
   return 0;
}

/*
 * Tries to cancel a command and put the camera into a kind of pristine state 
 */
int cancel(int cam)
{
   uchar sendme[] = { 0x00, 0x9e, 0x00, 0x00 }, *resblock;
   int res, errcode, len, dumint;
   int cumtime = 0;
   int c;

   fprintf(stderr, "Trying to clean up, please stand by...\n");
   DEBUG(stderr, "Waiting for ACK\n");
   while ((cumtime += 100) < 4000) {
      camputc(cam, ENQ);
      do
	 c = camgetc(cam, 1);
      while (c != ACK && c != -1);
      PROTOSNOOP(stderr, '\n', '\n');
      if (c == ACK)
	 break;
      usleep(100000L);
   }
   if (cumtime > 4000)
      return -1;
   res = camsenddata(cam, sendme, 4);
   if (res)
      return -1;
   resblock = get_datablock(cam, &len, &dumint, 0, 0);
   if (resblock == NULL || resblock == (void *) (-1))
      return -1;
   errcode = resblock[2] + (int) resblock[3] * 0x100;
   if (errcode) {
      cam_err(errcode);
      return -1;
   }
   return 0;
}

int send_command(int cam, uchar * datablock, int len,
		 int init_timeout, uchar ** infoblock,
		 int getsecondblock)
{
   int res, errcode, rlen, dumint;
   uchar *zw;
   int rtval = 0;

   if (!init_timeout)
      init_timeout = TIMEOUT_EOT;
   res = camsenddata(cam, datablock, len);
   if (res < 0) {
      fprintf(stderr, "failed -- camera did not accept command\n");
      return -1;
   }
   if (getsecondblock) {
      zw = get_datablock(cam, &rlen, &dumint, 0, init_timeout);
      if (!zw) {
	 fprintf(stderr, "\nResponse block could not be read\n");
	 return -1;
      }
      *infoblock = zw;
   }
   zw = get_datablock(cam, &rlen, &dumint, 0, init_timeout);
   if (!zw || zw == (void *) (-1)) {
      fprintf(stderr, "\nInfo/Response block could not be read\n");
      return -1;
   }
   errcode = zw[2] + (unsigned int) zw[3] * 0x100;
   /* || 10 added by JS for C30 support; might want to figure out
      why sometime */
   if (rlen == 6 || rlen == 10) {
/* do we never want to add zw[5] ?? */
      rtval = zw[4];		/*
				 * + (unsigned int)zw[5]*0x100; 
				 */
   }
   if (!getsecondblock && infoblock)
      *infoblock = zw;
   else
      free(zw);
   if (errcode) {
      fprintf(stderr, "failed -- ");
      cam_err(errcode);
      return -1;
   }
   return rtval;
}

/* Markus: does this do anything on your camera? */
void getinfo(int cam)
{
   uchar sendme[] = { 0x10, 0x90, 0x0, 0x0, 0x0, 0x0 };
   uchar *info;
   char buffer[30];

   send_command(cam, sendme, 6, 0, &info, 0);

   decode_str(buffer, 4, 6, info);
   printf("EISA id is: %s\n", buffer);
   decode_str(buffer, 8, 11, info);
   printf("Product id is: %s\n", buffer);
   decode_str(buffer, 12, 21, info);
   printf("Serial number: %s\n", buffer);
   printf("Hardware version number: %d.%d\n", (int) info[22],
	  (int) info[23]);
   printf("Software version of main CPU: %d.%d\n", (int) info[24],
	  (int) info[25]);
   printf("Software version of slave CPU: %d.%d\n", (int) info[26],
	  (int) info[27]);
   decode_str(buffer, 28, 48, info);
   printf("Product name: %s\n", buffer);
   decode_str(buffer, 50, 76, info);
   printf("Copyright: %s\n", buffer);
   free(info);
   return;
}

int setpref(int cam, unsigned int preffield, unsigned int prefvalue)
{
   uchar sendme[] =
       { 0xc0, 0x90, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };

   sendme[4] = preffield & 0xff;
   sendme[5] = (preffield & 0xff00) >> 8;
   sendme[6] = prefvalue & 0xff;
   sendme[7] = (prefvalue & 0xff00) >> 8;
   DEBUG(stderr, "Setpref %x, %x\n", preffield, prefvalue);
   return send_command(cam, sendme, 8, 0, NULL, 0);
}

/*
 * Gets the image info for for image number imgno, returns image ID 
 */
int getimageinfo(int imgno, int cam, uchar ** resblock)
{
   uchar sendme[] =
       { 0x20, 0x88, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00 };
   int picid;

   sendme[6] = imgno;
   picid = send_command(cam, sendme, 8, 0, resblock, 1);
   if (!picid) {
      DEBUG(stderr, "Getimageinfo failed for imgno %x\n", imgno);
      return -1;
   }
   return picid;
}

/*
 * Set time and date 
 */
int set_tod(int cam)
{
   uchar sendme[] = { 0xb0, 0x90, 0x00, 0x00,
      0x00,			/*
				 * year 
				 */
      0x00,			/*
				 * month 
				 */
      0x00,			/*
				 * day 
				 */
      0x00,			/*
				 * hours 
				 */
      0x00,			/*
				 * minutes 
				 */
      0x00			/*
				 * seconds 
				 */
   };
   struct tm *unixtime;
   time_t t;

   DEBUG(stderr, "Setting time\n");
   time(&t);
   unixtime = localtime(&t);
   sendme[4] = unixtime->tm_year % 100;
   sendme[5] = unixtime->tm_mon + 1;
   sendme[6] = unixtime->tm_mday;
   sendme[7] = unixtime->tm_hour;
   sendme[8] = unixtime->tm_min;
   sendme[9] = unixtime->tm_sec;
   return send_command(cam, sendme, 10, 0, NULL, 0);
}

/*
 * gets the camera's status -- right now, I only use it to enquire the number 
 * of pictures in the camera, and I return this number 
 */
int get_status(int cam)
{
   uchar sendme[] = { 0x20, 0x90, 0x00, 0x00, 0x00, 0x00 };
   uchar *dir;
   int res;

   res = send_command(cam, sendme, 6, 4000, &dir, 0);
   if (res < 0)
      return -1;
   res = dir[12];		/*
				 * + dir[13] 0x100; 
				 */
   free(dir);
   return res;
}

void decode_str(char *buf, int start, int end, uchar * block)
{
   int i;

   for (i = start; i <= end; i++) {
      buf[i - start] = (int) block[i];
   }
   buf[i - start] = '\0';
   return;
}

void cleanup(int signo)
{
   signo = signo + 1;
   if (global_cam > 0) {
      cancel(global_cam);
      cam_close(global_cam);
   }
   exit(-1);
}

void usage(char *progname,char *proghelp,char *moreopts)
{
    fprintf(stderr,proghelp,progname);
    fprintf(stderr,
    	"  -h this help\n"\
    	"  -r <bitrate> speed of connection (also PSM_BPS)\n"\
        "  -p <port> device file to read from (also PSM_PORT)\n"\
	"  -t sync camera's clock with host's clock\n");
    fprintf(stderr,moreopts);
}

/* Hello,vim...
vi: ts=8
*/
