#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>
#include <malloc.h>
#include "psm.h"
#include "config.h"

int parse_opts(int argc, char **argv, long *bitrate, char *camdev,
	       int *settime, int *isimgid)
{
   char optstr[] = "hitp:r:";
   int curopt;
   extern char *optarg;

   while ((curopt = getopt(argc, argv, optstr)) != EOF) {
      switch (curopt) {
      case 'i':
      	 *isimgid = 1;
	 break;
      case 'r':
	 sscanf(optarg, "%ld", bitrate);
	 break;
      case 'p':
	 strncpy(camdev, optarg, 69);
	 break;
      case 't':
	 *settime = 1;
	 break;
      case 'h':
      default:
      	 return -1;
      }
   }
   return 0;
}


/*
 * Deletes picture with image ID picno
 */
int delpic(int picno, int cam)
{
   uchar sendme[] =
       { 0x00, 0x80, 0x00, 0x00, 0x02, 0x00, 0x00, 0x80 };
   int dummy, res, dumint;
   uchar *zw;
   int errcode;

   sendme[6] = picno;
   res = camsenddata(cam, sendme, 8);
   if (res < 0) {
      fprintf(stderr, "failed -- camera did not accept command\n");
      return -1;
   }
   zw = get_datablock(cam, &dummy, &dumint, 0, 20000);
   if (!zw) {
      fprintf(stderr,
	      "Command timed out, but might still be completed\n");
      return -1;
   }
   errcode = zw[2] + (int) zw[3] * 0x100;
   free(zw);
   return errcode;
}

int delpic_imno(int picno, int cam)
{
   int imid;
   uchar *picdesc;

   imid = getimageinfo(picno, cam, &picdesc);
   if (imid == -1)
      return -1;
   free(picdesc);
   return delpic(imid, cam);
}

/*
 * Deletes all pictures in the camera
 */
int delall(int cam)
{
   uchar sendme[] = { 0x10, 0x80, 0x00, 0x00, 0x02, 0x00 };
   int retries = 0, len, dumint, errcode;
   uchar *res;

   while (retries++ < 3) {
      if (!camsenddata(cam, sendme, 6))
	 break;
   }
   if (retries == 4)
      return -1;
   res = get_datablock(cam, &len, &dumint, 0, 20000);
   if (!res) {
      fprintf(stderr, "Camera did not respond properly");
      return -1;
   }
   errcode = res[2] + res[3] * 0x100;
   if (errcode)
      cam_err(errcode);
   free(res);
   return errcode ? -1 : 0;
}

void this_usage(char *progname)
{
    usage(progname,"%s [opts] {<imgno>}|all: get pictures"\
    	" from HP Photosmart cameras\n",
    	"  -i numbers are image ids (not image numbers -- don't worry)");
    exit(-1);
}

int main(int argc, char **argv)
{
   int cam, i, npic;
   char camdev[70] = DEFAULT_PORT;
   long bitrate = DEFAULT_BITRATE;
   extern int optind;
   int settime = 0;
   int isimgid = 0;

   signal(SIGINT, cleanup);
   signal(SIGHUP, cleanup);
   signal(SIGTERM, cleanup);
   if (getenv("PSM_BPS")) {
      sscanf(getenv("PSM_BPS"), "%ld", &bitrate);
      if ((bitrate != 19200) && (bitrate != 38400) &&
	  (bitrate != 57600) && (bitrate != 115200)) {
	 fprintf(stderr, "Unknown bitrate %ld, using 19200\n",
		 bitrate);
	 bitrate = 19200;
      }
   }
   if (getenv("PSM_PORT"))
      strncpy(camdev, getenv("PSM_PORT"), 69);
   if (parse_opts(argc, argv, &bitrate, camdev, &settime, &isimgid)) {
      this_usage(argv[0]);
   }
   if (argc <= optind) {
      this_usage(argv[0]);
   }
   cam = cam_open(camdev, bitrate);
   if (cam < 0) {
      fprintf(stderr,
	      "Camera failed to acknowledge ping.  Is it connected?\n");
      return -1;
   }
   if (cam == 0) {
      fprintf(stderr, "Error opening serial port.  Is is busy?\n");
      return -1;
   }
   if (settime)
      set_tod(cam);

   if (!strcmp(argv[optind], "all")) {
      printf("Deleting all pictures...\n");
      delall(cam);
      return 0;
   }
   for (i = optind; i < argc; i++) {
      if (!sscanf(argv[i], "%d", &npic)) {
	 fprintf(stderr,
		 "Problem figuring out picture number from \"%s\", skipping.\n",
		 argv[i]);
	 continue;
      }
      printf("Deleting picture %d ...\n", npic);
      if ((isimgid?delpic(npic,cam):delpic_imno(npic, cam)) != 0) {
	 fprintf(stderr, "Problem deleting picture number %d\n",
		 npic);
      }
   }
   cam_close(cam);
   return 0;
}
/*
vi: ts=8
*/
