#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>
#include <malloc.h>
#include <math.h>
#include "psm.h"
#include "config.h"
#include "exif.h"

int
parse_opts(int argc, char **argv, long *bitrate, char *camdev,
	   char *prefix, int *settime,int *do_getinfo)
{
   char optstr[] = "whn:p:r:t";
   int curopt;
   extern char *optarg;

   while ((curopt = getopt(argc, argv, optstr)) != EOF) {
      switch (curopt) {
      case 'r':
	 sscanf(optarg, "%ld", bitrate);
	 break;
      case 'p':
	 strncpy(camdev, optarg, 69);
	 break;
      case 'n':
	 strncpy(prefix, optarg, 19);
	 break;
      case 't':
	 *settime = 1;
	 break;
      case 'w':
      	 *do_getinfo = 1;
	 break;
      case 'h':
      default:
	 return -1;
      }
   }
   return 0;
}

int dirall(int cam, char *prefix)
{
   int nopics, i;
   int ind;
   uchar *picdesc;
   char *resstr, *dateorig, *cp;
   exif_properties_t prop;
   double compression;

   if (!strcmp(prefix, ""))
      strcpy(prefix, "pic");

   nopics = get_status(cam);
   printf("%22s - %6s - %-19s\n",
	  "filename", "res.", "    Date/time");
   printf("-----------------------------------------------------\n");
   for (i = 1; i <= nopics; i++) {
      ind = getimageinfo(i, cam, &picdesc);
      if (ind == -1) {
	 fprintf(stderr, "No image info found -- bailing out\n");
	 return -1;
      }
      if (exif_get_properties((uint8_t *) picdesc, 200, &prop)) {
	 printf("Exif format error\n");
	 continue;
      }
      compression =
	  exif_get_rational(EXIF_COMPRESSEDBITSPERPIXEL, &prop);
      if (fabs(compression - 10) < 0.1 || fabs(compression-3.6)<0.01)
	 resstr = "High";
      else if (fabs(compression - 2) < 0.1 || fabs(compression-1.5)<0.01)
	 resstr = "Medium";
      else if (fabs(compression - 1) < 0.1 || fabs(compression-3.1)<0.01)
	 resstr = "Low";
      else
	 resstr = "Nonstandard";
/* for Jeff, 3.1 == low, 1.5 == medium, 3.6 == high resolution */
/* Are you absolutely sure?  That would point to a serious bug with me... */
      if (!(cp = exif_get_string(EXIF_DATETIMEORIGINAL, &prop)))
	 dateorig = "Date not accessible";
      else {
	 dateorig = cp;
	 dateorig[4] = '/';
	 dateorig[7] = '/';
      }
      printf("%16s%02d.jpg - %6s - %19s\n", prefix, i,
	     resstr, dateorig);
      if (cp)
	 free(cp);
      exif_release_properties(&prop);

      free(picdesc);
   }
   return 0;
}

void this_usage(char *progname)
{
    usage(progname,"%s [opts] {<picno>}: get camera directory"\
    	" from HP Photosmart cameras\n",
    	"  -w show camera info\n");
    exit(-1);
}
    
int main(int argc, char **argv)
{
   int cam, npic;
   char camdev[70] = DEFAULT_PORT;
   long bitrate = DEFAULT_BITRATE;
   char prefix[20] = "";
   int settime = 0,do_getinfo = 0;

   signal(SIGINT, cleanup);
   signal(SIGHUP, cleanup);
   signal(SIGTERM, cleanup);
   if (getenv("PSM_BPS")) {
      sscanf(getenv("PSM_BPS"), "%ld", &bitrate);
      if ((bitrate != 19200) && (bitrate != 38400) &&
	  (bitrate != 57600) && (bitrate != 115200)) {
	 fprintf(stderr, "Unknown bitrate %ld, using 19200\n",
		 bitrate);
	 bitrate = 19200;
      }
   }
   if (getenv("PSM_PORT"))
      strncpy(camdev, getenv("PSM_PORT"), 69);
   if (parse_opts(argc, argv, &bitrate, camdev, prefix, &settime, 
   	&do_getinfo)) {
      this_usage(argv[0]);
   }
   cam = cam_open(camdev, bitrate);
   if (cam < 0) {
      fprintf(stderr,
	      "Camera failed to acknowledge ping.  Is it connected?\n");
      return -1;
   }
   if (cam == 0) {
      fprintf(stderr, "Error opening serial port.  Is is busy?\n");
      return -1;
   }

   if (settime)
      set_tod(cam);

   if (do_getinfo) {
      getinfo(cam);
      return 0;
   }
   npic = get_status(cam);
   if (npic == 1)
      fprintf(stderr, "There is 1 image in the camera.\n");
   else
      fprintf(stderr, "There are %d images in the camera.\n", npic);
   dirall(cam, prefix);
   cam_close(cam);
   return 0;
}
/* Hi vim,
vi: ts=8
*/
