#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include "psm.h"
#include "config.h"

int parse_opts(int argc, char **argv, char *prefix, long *bitrate,
	       char *camdev, int *settime, int *flash)
{
   char optstr[] = "fghp:r:t";
   int curopt;
   extern char *optarg;

   while ((curopt = getopt(argc, argv, optstr)) != EOF) {
      switch (curopt) {
      case 'r':
	 sscanf(optarg, "%ld", bitrate);
	 break;
      case 'p':
	 strncpy(camdev, optarg, 69);
	 break;
      case 'f':
	 *flash = 1;
	 break;
      case 't':
	 *settime = 1;
	 break;
      case 'h':
      default:
         return -1;
      }
   }
   return 0;
}

/*
 * Get a picture
 */
int shootpic(int cam)
{
   uchar sendme[] = { 0x00, 0x91, 0x00, 0x00, 0x02, 0x00 };
   int imgid;
   uchar *infoblock;

   imgid = send_command(cam, sendme, 6, 10000, &infoblock, 1);
   if (imgid < 0)
      return -1;
   return imgid;
}


void this_usage(char *progname)
{
    usage(progname,"%s [opts]: take a picture"\
    	" with a HP Photosmart camera\n",
    	"  -f don't disable flash\n"\
	"returns image id of the picture just taken\n");
    exit(-1);
}
    

int main(int argc, char **argv)
{
   int cam;
   char camdev[70] = DEFAULT_PORT;
   char prefix[20] = "";
   long bitrate = DEFAULT_BITRATE;
   int settime = 0, flash = 0;
   int rtval;

   signal(SIGINT, cleanup);
   signal(SIGHUP, cleanup);
   signal(SIGTERM, cleanup);
   if (getenv("PSM_BPS")) {
      sscanf(getenv("PSM_BPS"), "%ld", &bitrate);
      if ((bitrate != 19200) && (bitrate != 38400) &&
	  (bitrate != 57600) && (bitrate != 115200)) {
	 fprintf(stderr, "Unknown bitrate %ld, using 19200\n",
		 bitrate);
	 bitrate = 19200;
      }
   }
   if (getenv("PSM_PORT"))
      strncpy(camdev, getenv("PSM_PORT"), 69);
   if (parse_opts
       (argc, argv, prefix, &bitrate, camdev, &settime, &flash)) {
       this_usage(argv[0]);
   }
   cam = cam_open(camdev, bitrate);
   if (cam < 0) {
      fprintf(stderr,
	      "Camera failed to acknowledge ping.  Is it connected?\n");
      return -1;
   }
   if (cam == 0) {
      fprintf(stderr, "Error opening serial port.  Is is busy?\n");
      return -1;
   }

   if (settime)
      set_tod(cam);

   if (!flash && setpref(cam, PREF_FLASH, 0))
      fprintf(stderr, "Could not turn off flash\n");
   rtval = shootpic(cam);
   cam_close(cam);
   return rtval;
}
/*
vi: ts=8
*/
